<?php

/*
 * Adds the dashboard widget. Dashboard widget is created in a function called
 * lbakut_widget() later in this file.
*/
function  lbakut_dashboard_setup() {
    $options = lbakut_get_options();
    if ($options['widget_show'] == true) {
        wp_add_dashboard_widget('lbakut_widget',
                __('User Tracking', 'lbakut'), 'lbakut_widget');
    }
    if ($options['stats_browser_widget'] == true) {
        wp_add_dashboard_widget('lbakut_browser_widget',
                __('Browser Stats', 'lbakut'), 'lbakut_browser_widget');
    }
    if ($options['stats_os_widget'] == true) {
        wp_add_dashboard_widget('lbakut_os_widget',
                __('OS Stats', 'lbakut'), 'lbakut_os_widget');
    }
    if ($options['stats_pageviews_widget'] == true) {
        wp_add_dashboard_widget('lbakut_pageviews_widget',
                __('Page View Stats', 'lbakut'), 'lbakut_pageviews_widget');
    }
}

/*
 * Stats widgets functions.
 */
function lbakut_browser_widget() {
    $stat = 'browser';
    echo '<p>This is a pie chart generated by the LBAK User Tracking plugin
        using the Google Charts API to give you a breakdown of what browsers
        your users are using. Please note that this chart is based on page
        clicks, not unique visitors. To hide this widget, <a href="tools.php?page=lbakut#stats_settings">click here</a>.
        To view the stats page, <a href="tools.php?page=lbakut&step=stats">click here</a>.</p>';
    echo '<br />';
    echo '<img src="'.lbakut_get_chart(lbakut_chart_type($stat)).'" />';
}
function lbakut_os_widget() {
    $stat = 'os';
    echo '<p>This is a pie chart generated by the LBAK User Tracking plugin
        using the Google Charts API to give you a breakdown of what operating systems
        your users are using. Please note that this chart is based on page
        clicks, not unique visitors. To hide this widget, <a href="tools.php?page=lbakut#stats_settings">click here</a>.
        To view the stats page, <a href="tools.php?page=lbakut&step=stats">click here</a>.</p>';
    echo '<br />';
    echo '<img src="'.lbakut_get_chart(lbakut_chart_type($stat)).'" />';
}
function lbakut_pageviews_widget() {
    $stat = 'pageviews';
    echo '<p>This is a pie chart generated by the LBAK User Tracking plugin
        using the Google Charts API to give you a breakdown of what pages
        your users are viewing. Please note that this chart is based on page
        clicks, not unique visitors. To hide this widget, <a href="tools.php?page=lbakut#stats_settings">click here</a>.
        To view the stats page, <a href="tools.php?page=lbakut&step=stats">click here</a>.</p>';
    echo '<br />';
    echo '<img src="'.lbakut_get_chart(lbakut_chart_type($stat)).'" />';
}

/*
 * This function creates the link to the admin settings panel. It also links
 * the lbak_add_scripts() function to the loading of the admin panel page
 * through the admin_print_scripts-% hook.
*/
function lbakut_admin_menu() {
    $page = add_submenu_page('tools.php', 'LBAK User Tracking Options',
            'User Tracking', 'manage_options', 'lbakut', 'lbakut_menu_options');
    add_action('admin_print_scripts-' . $page, 'lbakut_add_scripts');
}



function lbakut_widget() {

    //Declare global variables.
    global $wpdb;

    //Get lbakut options.
    $options = lbakut_get_options();

    //Create comma separated lists for ip addresses and user ids for
    //database use.
    $ips = "'" . implode("', '", array_filter($options['widget_ignored_ips'])) . "'";
    $user_ids = implode(", ", array_filter($options['widget_ignored_users'], 'intval'));

    //If the ip or user id lists are empty give them value that won't upset the query.
    if ($ips == "''" || $ips == "'Add ignored IP addresses in here.'") {
        $ips = "'0.0.0.0'";
    }
    if ($user_ids == '' || $user_ids == "Add ignored user IDs in here.") {
        $user_ids = '-1';
    }

    //Get the part of the query that will ignore the admin user.
    if ($options['widget_ignore_admin']) {
        $ignore_admin = 'AND `user_level`!=10';
    }
    else {
        $ignore_admin = '';
    }

    //get the number of results to display.
    if (!isset($options['widget_no_to_display'])) {
        $options['widget_no_to_display'] = 10;
    }
    else {
        $options['widget_no_to_display'] = intval($options['widget_no_to_display']);
    }

    //Get rows from the database based on the options set in the admin panel
    $query = 'WHERE `ip_address` NOT IN ('.$ips.')
            '.$ignore_admin.'
            AND `user_id` NOT IN ('.$user_ids.')';

    echo '<p>To edit the style and behaviour of this widget,
        <a href="tools.php?page=lbakut">click here</a> or to search these
        results, <a href="tools.php?page=lbakut&step=search">click here</a>.</p>';

    echo lbakut_print_table($query, 'widget', $options);
}

/*
 * Standard format for displaying time and dates in lbakut.
*/
function lbakut_time_format($time) {
    return strftime("%l%P, %a %e %b, %Y", $time);
}

/*
 * A 'time ago' function. Displays how long ago a unix timestamp occured in
 * relation to time() (now). If it was over a week ago, the function just
 * returns the time and date formatted by lbakut_time_format().
*/
function lbakut_time_ago_format($time) {
    $temp = time() - $time;
    if ($temp > (60*60*24*7)) {
        return lbakut_time_format($time);
    }

    $days = floor($temp / 86400);
    $temp -= ($days * 86400);
    $hours = floor($temp / 3600);
    $temp -= ($hours * 3600);
    $mins = floor($temp / 60);
    $temp -= ($mins * 60);
    $secs = $temp;

    if ($days) {
        return $days . "d ago";
    }
    if ($hours) {
        return $hours . "h ago";
    }
    if ($mins) {
        return $mins . "m ago";
    }
    return $secs . "s ago";

    /*
    if ($days) {
        $length .= $days . "d ";
    }
    if ($hours) {
        $length .= $hours . "h ";
    }
    if ($mins) {
        $length .= $mins . "m ";
    }
    return $length .= $secs . "s ago";
    */
}

/*
 * Function used to translate options in the database
 * associative array (list available in housekeeping.php) into a human readable
 * format.
*/
function lbakut_option_translate($k) {
    switch($k) {

        //TRACKER SETTINGS
        case 'track_ip': return 'IP Address';
        case 'track_real_ip': return 'Real IP Address';
        case 'track_referrer': return 'Referrer';
        case 'track_time': return 'Time';
        case 'track_user_id': return 'User ID';
        case 'track_user_level': return 'User Level';
        case 'track_display_name': return 'Display Name';
        case 'track_user_agent': return 'User Agent';
        case 'track_script_name': return 'Script Name';
        case 'track_query_string': return 'GET vars';
        case 'track_post_vars': return 'POST vars';
        case 'track_cookies': return 'Cookies';
        case 'track_ignore_admin': return '<b>Don\'t track admins</b>';

        //WIDGET SETTINGS
        case 'widget_show': return 'widget';
        case 'widget_show_referrer': return 'Referrer';
        case 'widget_show_name': return 'Name';
        case 'widget_show_ip': return 'IP Address';
        case 'widget_show_real_ip': return 'Real IP Address';
        case 'widget_show_page': return 'Page';
        case 'widget_show_browser': return 'Browser';
        case 'widget_show_os': return 'OS';
        case 'widget_show_time': return 'Time Ago';
        case 'widget_show_get': return 'GET vars';
        case 'widget_show_post': return 'POST vars';
        case 'widget_show_cookies': return 'Cookies';
        case 'widget_show_page_title': return 'Page Title';

        //SEARCH SETTINGS
        case 'search_show_ip': return 'IP Address';
        case 'search_show_real_ip': return 'Real IP Address';
        case 'search_show_referrer': return 'Referrer';
        case 'search_show_time': return 'Time';
        case 'search_show_user_id': return 'User ID';
        case 'search_show_user_level': return 'User Level';
        case 'search_show_name': return 'Display Name';
        case 'search_show_browser': return 'Browser';
        case 'search_show_os': return 'OS';
        case 'search_show_page': return 'Script Name';
        case 'search_show_get': return 'GET vars';
        case 'search_show_post': return 'POST vars';
        case 'search_show_cookies': return 'Cookies';
        case 'search_show_page_title': return 'Page Title';

        //DEFAULT
        default: return $k;
    }
}

/*
 * Settings page descriptions based on the name of the options in the database
 * associative array (list available in housekeeping.php).
*/
function lbakut_option_description($k) {
    if (preg_match('/track_/', $k)) {
        $prefix = 'Check to track '.lbakut_option_translate($k).'.';
    }
    else if (preg_match('/search_/', $k)) {
        $prefix = 'Check to show '.lbakut_option_translate($k).' on the search page.';
    }
    else if (preg_match('/widget_show_/', $k)) {
        $prefix = 'Check to show '.lbakut_option_translate($k).' in the dashboard widget.';
    }
    else {
        $prefix = 'Check to track '.lbakut_option_translate($k).'.';
    }
    switch($k) {

        //TRACKER SETTINGS
        case 'track_real_ip': return $prefix.'.
            This attempts to get a user\'s real IP address if they are browsing
            through a proxy, however, the success rate is very low. Most good
            proxies will mask the IP address properly.';
        case 'track_post_vars': return $prefix.'.
            POST variables are variables sent to web servers in forms and such.';
        case 'track_user_agent': return $prefix.'.
            <b>NOTE: This is required for browser and OS detection.</b>';
        case 'track_ignore_admin': return 'Check this to disable the tracking
            of admin users.';

        //WIDGET SETTINGS
        case 'widget_show_real_ip': return $prefix.'
                    column on the admin panel widget. This field will only show an
                    ip address if the user has viewed your site through some sort
                    of proxy (perhaps even without their knowledge). Not 100% reliable.';
        case 'widget_show_referrer': return $prefix.'
                    column on the admin panel widget. There are a lot of ways to modify
                    the referrer variable so this field may not always be trustable.';
        case 'widget_show': return $prefix.'
                    on the admin panel widget.';
        default: return $prefix.'
                    column on the admin panel widget.';

        //SEARCH SETTINGS

        //DEFAULT
        default: return $prefix;
    }
}

/*
 * This function replaces all of the & symbols in a query string with line
 * breaks and bolds the variable name. Vastly increases readability.
*/
function lbakut_query_string_formatter($string) {
    //Decode html special chars (&, <, >, and quotes).
    $string = htmlspecialchars_decode($string);
    //Decode the URL chars (the ones with a % in front of them)
    $string = urldecode($string);
    //Corrects what seems to be a bug in how query strings are handled with
    //html encoding.
    $string = str_replace('amp;', '', $string);
    return preg_replace('/(.*?)=(.*?)(?:&|$)/i', '<b>$1</b>=$2<br />', $string);
}

/*
 * Function to show all of the options that start with $prefix correctly
 * formatted as table rows and part of a form. Used in the admin panel page to
 * change the plugin settings.
*/
function lbakut_print_options($prefix, $options = null) {
    if ($options == null) {
        $options = lbakut_get_options();
    }
    $return = '';
    foreach ($options as $k => $v) {
        if (preg_match('/'.$prefix.'/',$k)) {

            if ($options[$k] == true) {
                $checked = 'checked';
            }
            else {
                $checked = '';
            }

            if ($prefix == 'track_') {
                if ($k == 'track_ignore_admin') {
                    $pre = '';
                }
                else {
                    $pre = 'Track';
                }
            }
            else if ($prefix == 'widget_show_' || $prefix == 'search_') {
                $pre = 'Show';
            }
            else {
                $pre = 'Enable';
            }

            $return .= '
            <tr>
                <td>
                    <label for="'.$k.'">
                        '.__($pre.' '.lbakut_option_translate($k).'?', 'lbakut').'
                    </label>
                </td>
                <td>
                    <input type="hidden" name="'.$k.'" value="0" />
                    <input type="checkbox" id="'.$k.'"
                    name="'.$k.'" value="1" '.$checked.' />
                </td>
                <td>
                    <p>'.lbakut_option_description($k).'</p>
                </td>
            </tr>';
        }
    }

    return $return;
}

/*
 * I decided to put this into a function to make it easier to keep the box
 * state through page reloads. It prints a dropdown box of 'times ago' with
 * their corresponding value in seconds.
*/
function lbakut_time_ago_select_box($get_var) {
    $times = array(
            'Seconds' => 1,
            'Minutes' => 60,
            'Hours' => 3600,
            'Days' => 86400,
            'Weeks' => 604800,
            'Months' => 2628000,
            'Years' => 31536000
    );

    $return = '<select name="'.$get_var.'">';
    foreach($times as $k=>$v) {
        if ($_GET[$get_var] == $v) {
            $selected = 'selected';
        }
        else {
            $selected = '';
        }
        $return .= '<option value="'.$v.'" '.$selected.'>'.$k.'</option>';
    }
    $return .= '</select>';
    return $return;
}

/*
 * This is in a function to make it easier for the box to keep its state
 * through page reloads. For use with the is/isn't dropdowns on the search page.
*/
function lbakut_is_isnt_box($get_var) {
    global $_GET;
    if (stripslashes($_GET[$get_var]) != "isn't") {
        $is = 'selected';
        $isnt = '';
    }
    else {
        $is = '';
        $isnt = 'selected';
    }

    return "<select name='$get_var'>
            <option value='is' $is>is</option>
            <option value=\"isn't\" $isnt>isn't</option>
        </select>";
}

/*
 * This function takes a list of comma separated values (cvs's) and turns them
 * into part of an SQL query for use in the search page for this plugin.
*/
function lbakut_search_var_prepare($get_var, $type) {

    global $wpdb;

    if (!empty($_GET[$get_var])) {
        //Create array out of comma separated values (csv).
        $csv_array = explode(",", $_GET[$get_var]);

        //Get the correct operator for the SQL query.
        if (stripslashes($_GET[$get_var.'_is']) != "isn't") {
            $operator = '=';
        }
        else {
            $operator = '!=';
        }

        //If there is more than 1 value in the array.
        if (sizeof($csv_array) > 1) {
            $return = 'AND (';
            foreach ($csv_array as $value) {
                if ($operator == '=') {
                    $andor = ' OR ';
                }
                else {
                    $andor = ' AND ';
                }

                //Prepare the value for database entry based on $type.
                if ($type == 'string') {
                    $prepared = $wpdb->escape(trim($value));
                }
                else {
                    $prepared = intval($value);
                }
                $return .= '`'.$get_var.'`'.$operator."
                                            '".$prepared."'".$andor;
            }
            $return = rtrim($return, $andor).')';
        }
        else {

            //Prepare the value for database entry based on $type.
            if ($type == 'string') {
                $prepared = $wpdb->escape(trim($_GET[$get_var]));
            }
            else {
                $prepared = intval($_GET[$get_var]);
            }
            $return = 'AND `'.$get_var.'`'.$operator."
                                        '".$prepared."'";
        }
    }
    else {
        $return = 'AND 1';
    }
    return $return;
}

/*
 * This is the function that prints the table of data out. The two required
 * parameters are $where_clause, the where clause of the query string to get
 * data from the database and $place, which is the place of
 * the table. This is for sifting through options to find out what columns to
 * show, that is all.
*/
function lbakut_print_table($where_clause, $place, $options = null, $brows = null) {
    global $wpdb;

    if ($options == null) {
        $options = lbakut_get_options();
    }
    if ($brows == null) {
        $brows = lbakut_get_browscap();
    }

    $no_of_results = $wpdb->get_var('SELECT COUNT(`id`) as `count` FROM
        '.$options['main_table_name'].' WHERE `id` IN (SELECT `id` FROM 
            '.$options['main_table_name'].' '.$where_clause.')');

    $rows = $wpdb->get_results('SELECT * FROM `'.$options['main_table_name'].'`
        '.$where_clause.'
            ORDER BY `time` DESC 
            LIMIT '.$options[$place.'_no_to_display'].'
            OFFSET '.((lbakut_page_var())-1)*$options[$place.'_no_to_display']);

    $return .= lbakut_do_pagination($no_of_results, $place, $options).'<br />';
    $return .= '<table class="widefat">';
    $return .= '<thead>';
    if ($options[$place.'_show_time'] == true)
        $return .= '<th>Time</th>';
    if ($options[$place.'_show_name'] == true)
        $return .= '<th>Name</th>';
    if ($options[$place.'_show_user_id'] == true)
        $return .= '<th>User ID</th>';
    if ($options[$place.'_show_user_level'] == true)
        $return .= '<th>User Level</th>';
    if ($options[$place.'_show_ip'] == true)
        $return .= '<th>IP</th>';
    if ($options[$place.'_show_real_ip'] == true)
        $return .= '<th>Real IP</th>';
    if ($options[$place.'_show_browser'] == true)
        $return .= '<th>Browser</th>';
    if ($options[$place.'_show_os'] == true)
        $return .= '<th>OS</th>';
    if ($options[$place.'_show_referrer'] == true)
        $return .= '<th>Referrer</th>';
    if ($options[$place.'_show_page'] == true)
        $return .= '<th>Page</th>';
    if ($options[$place.'_show_page_title'] == true)
        $return .= '<th>Page Title</th>';
    if ($options[$place.'_show_get'] == true)
        $return .= '<th>GET vars</th>';
    if ($options[$place.'_show_post'] == true)
        $return .= '<th>POST vars</th>';
    if ($options[$place.'_show_cookies'] == true)
        $return .= '<th>Cookies</th>';
    $return .= '</thead>';
    foreach ($rows as $row) {
        if ($row->user_agent) {
            $browscap = lbakut_browser_info($row->user_agent, $brows);
        }
        else {
            $browscap = null;
        }

        if ($options[$place.'_show_referrer'] == true)
            $referrer = parse_url($row->referrer);

        $return .= '<tr>';

        if ($options[$place.'_show_time'] == true)
            $return .= '<td>'.lbakut_time_ago_format($row->time).'</td>';
        if ($options[$place.'_show_name'] == true)
            $return .= '<td><a href="tools.php?page=lbakut&step=search&user_id='.$row->user_id.'"
                title="Search for activity from the user '.$row->display_name.'">
                    '.$row->display_name.'</a></td>';
        if ($options[$place.'_show_user_id'] == true)
            $return .= '<td><a href="tools.php?page=lbakut&step=search&user_id='.$row->user_id.'"
                title="Search for activity from the user '.$row->display_name.'">
                    '.$row->user_id.'</td>';
        if ($options[$place.'_show_user_level'] == true)
            $return .= '<td>'.$row->user_level.'</td>';
        if ($options[$place.'_show_ip'] == true)
            $return .= '<td><a href="tools.php?page=lbakut&step=search&ip_address='.$row->ip_address.'"
                title="Search for activity for the ip '.$row->ip_address.'">'.$row->ip_address.'</a></td>';
        if ($options[$place.'_show_real_ip'] == true)
            $return .= '<td><a href="tools.php?page=lbakut&step=search&real_ip_address='.$row->real_ip_address.'"
                title="Search for activity for the ip '.$row->real_ip_address.'">
                    '.$row->real_ip_address.'</a></td>';
        if ($options[$place.'_show_browser'] == true)
            $return .= '<td>'.$browscap['parent'].'</td>';
        if ($options[$place.'_show_os'] == true)
            $return .= '<td>'.$browscap['platform'].'</td>';
        if ($options[$place.'_show_referrer'] == true)
            $return .= $row->referrer ? '<td>'.$referrer['scheme'].'://'.$referrer['host'].$referrer['path'].'</td>' : '<td></td>';
        if ($options[$place.'_show_page'] == true)
            $return .= '<td>'.$row->script_name.'</td>';
        if ($options[$place.'_show_page_title'] == true)
            $return .= '<td>'.$row->page_title.'</td>';
        if ($options[$place.'_show_get'] == true)
            $return .= '<td>'.lbakut_query_string_formatter($row->query_string).'</td>';
        if ($options[$place.'_show_post'] == true)
            $return .= '<td>'.lbakut_query_string_formatter($row->post_vars).'</td>';
        if ($options[$place.'_show_cookies'] == true)
            $return .= '<td>'.lbakut_query_string_formatter($row->cookies).'</td>';

        $return .= '</tr>';
    }
    $return .= '</table>';

    $return .= '<br />'.lbakut_do_pagination($no_of_results, $place, $options);

    return $return;
}

/*
 * Get the variable used in pagination.
*/
function lbakut_page_var() {
    if (!isset($_GET['lbakut_page'])) {
        return 1;
    }
    else {
        return $_GET['lbakut_page'];
    }
}

/*
 * Generates the pagination based on the amount of results and the place of
 * the table. $place is used for deciding how many rows get shown, thus how
 * many pages there are.
*/
function lbakut_do_pagination($no_of_results, $place, $options = null) {
    if ($options == null) {
        $options = lbakut_get_options();
    }

    $pages = ceil($no_of_results/$options[$place.'_no_to_display']);

    if ($pages < 2) {
        return null;
    }

    $start = max(1, lbakut_page_var() - 3);
    $end = min($pages, lbakut_page_var() + 3);

    $result .= '<div class="lbakut_pages">';
    $result .= 'Showing results '.((lbakut_page_var()-1)*$options[$place.'_no_to_display']+1).'
        - '.(lbakut_page_var()*$options[$place.'_no_to_display']).' of '.$no_of_results.'<br /><br />';

    $uri = preg_replace('/&lbakut_page=[0-9]+/', '', $_SERVER['QUERY_STRING']);

    if ($start != 1) {
        if ($uri == '') {
            $page_uri = '?lbakut_page=1';
        }
        else {
            $page_uri = '?'.$uri.'&lbakut_page=1';
        }
        $result .= '<span class="lbakut_page"><a href="'.$page_uri.'">&nbsp;1&nbsp;</a></span> ... ';
    }

    for ($page = $start; $page <= $end; $page++) {
        if ($uri == '') {
            $page_uri = '?lbakut_page='.$page;
        }
        else {
            $page_uri = '?'.$uri.'&lbakut_page='.$page;
        }

        if ($page == lbakut_page_var()) {
            $result .= '<span class="lbakut_page_selected"><a href="'.$page_uri.'">&nbsp;'.$page.'&nbsp;</a></span>';
        }
        else {
            $result .= '<span class="lbakut_page"><a href="'.$page_uri.'">&nbsp;'.$page.'&nbsp;</a></span>';
        }
    }

    if ($end != $pages) {
        if ($uri == '') {
            $page_uri = '?lbakut_page='.$pages;
        }
        else {
            $page_uri = '?'.$uri.'&lbakut_page='.$pages;
        }
        $result .= ' ... <span class="lbakut_page"><a href="'.$page_uri.'">&nbsp;'.$pages.'&nbsp;</a></span>';
    }

    $result .= '</div>';

    return $result;
}

?>